@@ -19,8 +19,10 @@
 import logging
 import os
 import pickle
+import subprocess
 import sys
-from ctypes import byref, c_float, c_int, pointer
+import ctypes
+from ctypes import byref, c_float, c_int, pointer, util
 from pathlib import Path
 
 from gi.repository import GLib
@@ -58,7 +60,7 @@ if not sys.warnoptions:
 if sys.platform != "win32":
 	import fcntl
 
-n_version = "8.2.3" # Should also be bumped in pyproject.toml, extra/*.appdata.xml
+n_version = "9.0.0" # Should also be bumped in pyproject.toml, extra/*.appdata.xml
 t_version = "v" + n_version
 t_title = "Tauon"
 t_id = "tauonmb"
@@ -75,6 +77,17 @@ def open_discord() -> None:
 def open_github() -> None:
 	webbrowser.open("https://github.com/Taiko2k/Tauon/issues")
 
+def open_crash_log(path: Path) -> None:
+	try:
+		if sys.platform == "win32":
+			os.startfile(path)  # type: ignore[attr-defined]
+		elif sys.platform == "darwin":
+			subprocess.Popen(["open", str(path)])
+		else:
+			subprocess.Popen(["xdg-open", str(path)])
+	except Exception:
+		webbrowser.open(path.as_uri())
+
 def main() -> None:
 	"""Launch Tauon by means of importing t_main.py"""
 	from tauon.t_modules.t_main import main as t_main
@@ -222,17 +235,31 @@ if Path(user_directory / "x11").exists():
 	logging.debug("Forcing X11 due to user prefs")
 	os.environ["SDL_VIDEODRIVER"] = "x11"
 
-# We currently only properly package SDL3 on Windows and macOS, remove the if check when Linux is fixed
-#if sys.platform in ("win32", "darwin"):
-# os.environ["SDL_BINARY_PATH"]              = "." # Set the path to your binaries,               "sdl3/bin" by default.
-if str(install_directory).startswith("/app/"):
-	os.environ["SDL_BINARY_PATH"] = "/app/lib"
-	# Disabling causes crash with AUR package as it ships docs and disabling them attempts a removal on system libraries
-	# Enabling crashes our Flatpak builds as we don't ship the docs there and it attempts a generation on a read-only fs
-	# So just disable it on Flatpak until https://github.com/Aermoss/PySDL3/issues/22 is resolved
-	os.environ["SDL_DOC_GENERATOR"]            = "0" # Disable doc generation                       "1"        by default.
+# Pysdl3 doesn't seem to find system sdl3. REF: https://github.com/Aermoss/PySDL3/issues/35 pysdl
+# As a workaround we try find it ourselves
+sdl_dll_name = ctypes.util.find_library("SDL3")
+if sdl_dll_name:
+	candidates = [
+		"./" + sdl_dll_name,
+		os.path.join("/app/lib", sdl_dll_name),
+		os.path.join("/usr/lib", sdl_dll_name),
+		os.path.join("/usr/lib64", sdl_dll_name),
+		os.path.join("/lib", sdl_dll_name),
+		os.path.join("/lib64", sdl_dll_name),
+		os.path.join("/usr/local/lib", sdl_dll_name),
+		os.path.join("/usr/local/lib64", sdl_dll_name),
+	]
+else:
+	candidates = []
+sdl_library_path = next((p for p in candidates if p and os.path.exists(p)), None)
+if sdl_library_path:
+	os.environ["SDL_BINARY_PATH"] = os.path.dirname(sdl_library_path)
+
+os.environ["SDL_FIND_BINARIES"] = "1"
 if pyinstaller_mode:
-	os.environ["SDL_FIND_BINARIES"]            = "0" # Search for binaries in the system libraries, "1"        by default.
+	os.environ["SDL_FIND_BINARIES"]            = "0" # Disable binary searching,                   "1"        by default.
+
+os.environ["SDL_DOC_GENERATOR"]            = "0" # Disable doc generation                       "1"        by default.
 os.environ["SDL_DISABLE_METADATA"]         = "1" # Disable metadata method,                     "0"        by default.
 os.environ["SDL_CHECK_VERSION"]            = "0" # Disable version checking,                    "1"        by default.
 os.environ["SDL_CHECK_BINARY_VERSION"]     = "0" # Disable binary version checking,             "1"        by default.
@@ -244,14 +271,7 @@ import sdl3  # noqa: E402
 sethint_result = sdl3.SDL_SetHint(sdl3.SDL_HINT_VIDEO_ALLOW_SCREENSAVER, b"1")
 if sethint_result is None:
 	logging.error("Failed to run SetHint, probably due to https://github.com/Aermoss/PySDL3/issues/35, will try a workaround")
-	os.environ["SDL_BINARY_PATH"] = "/usr/lib"
-	import importlib
-	importlib.reload(sdl3)
-
-	sethint_result = sdl3.SDL_SetHint(sdl3.SDL_HINT_VIDEO_ALLOW_SCREENSAVER, b"1")
-	if sethint_result is None:
-		logging.critical("Could not call SetHint, crashing")
-		sys.exit(1)
+	sys.exit(1)
 
 sdl3.SDL_SetHint(sdl3.SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, b"1")
 sdl3.SDL_SetHint(sdl3.SDL_HINT_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR, b"0")
@@ -422,6 +442,8 @@ sdl3.SDL_RenderClear(renderer)
 
 logging.info(f"SDL window system: {sdl3.SDL_GetCurrentVideoDriver().decode()}")
 
+sdl3.SDL_SetRenderVSync(renderer, 1) # 1 == enable vsync, 0 == disable, -1 == late swap tearing, 2 == second vertical refresh
+
 i_x = pointer(c_int(0))
 i_y = pointer(c_int(0))
 sdl3.SDL_GetWindowSizeInPixels(t_window, i_x, i_y)
@@ -516,12 +538,14 @@ if __name__ == "__main__":
 
 		BUTTON_ID_GITHUB = 1
 		BUTTON_ID_DISCORD = 2
-		BUTTON_ID_QUIT = 3
+		BUTTON_ID_OPEN_LOG = 3
+		BUTTON_ID_QUIT = 4
 
 		# Create buttons
-		button_array = (sdl3.SDL_MessageBoxButtonData * 3)(
+		button_array = (sdl3.SDL_MessageBoxButtonData * 4)(
 			sdl3.SDL_MessageBoxButtonData(0, BUTTON_ID_GITHUB, b"GitHub"),
 			sdl3.SDL_MessageBoxButtonData(0, BUTTON_ID_DISCORD, b"Discord"),
+			sdl3.SDL_MessageBoxButtonData(0, BUTTON_ID_OPEN_LOG, b"Open Crash Log"),
 			sdl3.SDL_MessageBoxButtonData(sdl3.SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT, BUTTON_ID_QUIT, b"Quit"),
 		)
 		messageboxdata = sdl3.SDL_MessageBoxData(
@@ -529,7 +553,7 @@ if __name__ == "__main__":
 			window=None,
 			title=b"Tauon Music Box crashed :(",
 			message=str.encode(f"{error_message}\n\nShortlog:\n{e}"),
-			numbuttons=3,
+			numbuttons=4,
 			buttons=button_array,
 			color_scheme=None,
 		)
@@ -544,5 +568,7 @@ if __name__ == "__main__":
 			open_discord()
 		elif buttonid.value == BUTTON_ID_GITHUB:
 			open_github()
+		elif buttonid.value == BUTTON_ID_OPEN_LOG:
+			open_crash_log(crash_log_path)
 		elif buttonid.value == BUTTON_ID_QUIT:
 			pass
