@@ -22,8 +22,7 @@ import ctypes
 import io
 import logging
 import math
-import sys
-from ctypes import c_bool, c_float, byref, c_size_t, pointer
+from ctypes import byref, c_bool, c_float, c_size_t, pointer
 from typing import TYPE_CHECKING
 
 import sdl3
@@ -46,31 +45,19 @@ except Exception:
 	logging.exception("Unknown error trying to import jxlpy, JPEG XL support will be disabled.")
 
 
-system = "Linux"
+import sys
+
 if sys.platform == "win32":
-	system = "Linux"  # "windows"
 	import os
 
 	os.environ["PANGOCAIRO_BACKEND"] = "fc"
 
-if system == "Linux":
-	import struct
+import cairo
+import gi
 
-	import cairo
-	import gi
-
-	gi.require_version("Pango", "1.0")
-	gi.require_version("PangoCairo", "1.0")
-	from gi.repository import Pango, PangoCairo
-
-else:
-	import struct
-	from ctypes import CFUNCTYPE, POINTER, byref, c_void_p, windll
-
-	import win32api
-	import win32con
-	import win32gui
-	import win32ui
+gi.require_version("Pango", "1.0")
+gi.require_version("PangoCairo", "1.0")
+from gi.repository import Pango, PangoCairo
 
 
 class QuickThumbnail:
@@ -135,176 +122,6 @@ class QuickThumbnail:
 		return True
 
 
-# TODO(Martin): This block never executes - https://github.com/Taiko2k/Tauon/issues/1318
-if system == "Windows":
-
-	class RECT(ctypes.Structure):
-		_fields_ = [
-			("left", ctypes.c_long),
-			("top", ctypes.c_long),
-			("right", ctypes.c_long),
-			("bottom", ctypes.c_long),
-		]
-
-	def RGB(r: int, g: int, b: int) -> int:
-		return r | (g << 8) | (b << 16)
-
-	def Wcolour(colour: ColourRGBA) -> int:
-		return colour.r | (colour.g << 8) | (colour.b << 16)
-
-	def native_bmp_to_sdl(hdc, bitmap_handle, width: int, height: int):  # -> tuple[Unknown, Array[c_char]]
-		bmpheader = struct.pack("LHHHH", struct.calcsize("LHHHH"), width, height, 1, 24)  # w,h, planes=1, bitcount)
-
-		c_bmpheader = ctypes.c_buffer(bmpheader)
-
-		# 3 bytes per pixel, pad lines to 4 bytes
-		c_bits = ctypes.c_buffer(b" " * (height * ((width * 3 + 3) & -4)))
-
-		res = ctypes.windll.gdi32.GetDIBits(hdc, bitmap_handle, 0, height, c_bits, c_bmpheader, win32con.DIB_RGB_COLORS)
-
-		if not res:
-			raise OSError("native_bmp_to_pil failed: GetDIBits")
-
-		# TODO(Martin): Add the rest of the types in this function:
-		logging.debug(
-			f"IF YOU SEE THIS MESSAGE, ADD THESE TYPES TO native_bmp_to_sdl(): HDC: {type(hdc)}, bitmap_handle: {type(bitmap_handle)}, returnType:{type(sdl3.SDL_CreateSurfaceFrom(width, height, sdl3.SDL_PIXELFORMAT_BGR24, ctypes.pointer(c_bits), (width * 3 + 3) & -4))}"
-		)
-		# We need to keep c_bits pass else it may be garbage collected
-		return sdl3.SDL_CreateSurfaceFrom(
-			width, height, sdl3.SDL_PIXELFORMAT_BGR24, ctypes.pointer(c_bits), (width * 3 + 3) & -4
-		), c_bits
-
-	class Win32Font:
-		def __init__(
-			self,
-			name: str,
-			height: int,
-			weight: int = win32con.FW_NORMAL,
-			italic: bool = False,
-			underline: bool = False,
-		) -> None:
-			self.font = win32ui.CreateFont(
-				{"name": name, "height": height, "weight": weight, "italic": italic, "underline": underline}
-			)  #'charset': win32con.MAC_CHARSET})
-
-			# create a compatible DC we can use to draw:
-
-			self.desktopHwnd = win32gui.GetDesktopWindow()
-			self.desktopDC = win32gui.GetWindowDC(self.desktopHwnd)
-			self.mfcDC = win32ui.CreateDCFromHandle(self.desktopDC)
-			self.drawDC = self.mfcDC.CreateCompatibleDC()
-
-			# initialize it
-
-			self.drawDC.SelectObject(self.font)
-
-		def get_metrics(self, text: str, max_x: int, wrap: bool) -> tuple[int, int]:
-			# return self.drawDC.GetTextExtent(text)
-
-			rect = RECT(0, 0, 0, 0)
-			rect.left = 0
-			rect.right = round(max_x)
-			rect.top = 0
-			rect.bottom = 0
-
-			# windll.User32.DrawTextW(t, text, len(text)) #, rect, win32con.DT_WORDBREAK)
-			t = self.drawDC.GetSafeHdc()
-
-			if wrap:
-				windll.User32.DrawTextW(t, text, len(text), pointer(rect), win32con.DT_WORDBREAK | win32con.DT_CALCRECT)
-			else:
-				windll.User32.DrawTextW(
-					t, text, len(text), pointer(rect), win32con.DT_CALCRECT | win32con.DT_END_ELLIPSIS
-				)
-
-			return rect.right, rect.bottom
-
-		def renderText(
-			self,
-			text: str,
-			bg: ColourRGBA,
-			fg: ColourRGBA,
-			wrap: bool = False,
-			max_x: int = 100,
-			max_y: int | None = None,
-		):
-			self.drawDC.SetTextColor(Wcolour(fg))
-
-			t = self.drawDC.GetSafeHdc()
-
-			win32gui.SetBkMode(t, win32con.TRANSPARENT)
-
-			# create the compatible bitmap:
-
-			# w,h = self.drawDC.GetTextExtent(text)
-			w, h = self.get_metrics(text, max_x, wrap)
-
-			# logging.info(self.drawDC.GetTextFace())
-
-			# w += 1
-			# if wrap:
-			# 	h = int((w / max_x) * h) + h
-			# 	w = max_x + 1
-			if max_y is not None:
-				h = max_y
-
-			saveBitMap = win32ui.CreateBitmap()
-
-			saveBitMap.CreateCompatibleBitmap(self.mfcDC, w, h)
-
-			self.drawDC.SelectObject(saveBitMap)
-
-			# draw it
-
-			br = win32ui.CreateBrush(win32con.BS_SOLID, Wcolour(bg), 0)
-
-			self.drawDC.FillRect((0, 0, w, h), br)
-
-			# self.drawDC.DrawText(text, (0, 0, w, h), win32con.DT_LEFT)
-
-			# windll.gdi32.TextOutW(t, 0, 0, "test", 5)
-
-			if wrap:
-				rect = RECT(0, 0, 0, 0)
-				rect.left = 0
-				rect.right = round(max_x)
-				rect.top = 0
-				rect.bottom = round(h)
-
-				# windll.User32.DrawTextW(t, text, len(text)) #, rect, win32con.DT_WORDBREAK)
-				windll.User32.DrawTextW(t, text, len(text), pointer(rect), win32con.DT_WORDBREAK)
-			else:
-				rect = RECT(0, 0, 0, 0)
-				rect.left = 0
-				rect.right = round(max_x)
-				rect.top = 0
-				rect.bottom = round(h)
-
-				# windll.User32.DrawTextW(t, text, len(text)) #, rect, win32con.DT_WORDBREAK)
-				windll.User32.DrawTextW(t, text, len(text), pointer(rect), win32con.DT_END_ELLIPSIS)
-
-				# windll.gdi32.TextOutW(t, 0, 0, text, len(text))
-
-			# logging.info(rects)
-			# logging.info(text)
-			# windll.gdi32.ExtTextOutW(t, 0, 0, None, rect, text, len(text), None)
-			# convert to SDL surface
-			im, c_bits = native_bmp_to_sdl(self.drawDC.GetSafeHdc(), saveBitMap.GetHandle(), w, h)
-			# clean-up
-			win32gui.DeleteObject(saveBitMap.GetHandle())
-
-			return im, c_bits
-
-		def __del__(self) -> None:
-			self.mfcDC.DeleteDC()
-			self.drawDC.DeleteDC()
-			win32gui.ReleaseDC(self.desktopHwnd, self.desktopDC)
-			win32gui.DeleteObject(self.font.GetSafeHandle())
-
-		def __del__(self) -> None:
-			win32gui.DeleteObject(self.font.GetSafeHandle())
-
-
 perf = Timer()
 
 
@@ -322,25 +139,16 @@ class TDraw:
 		self.source_rect = sdl3.SDL_FRect(0.0, 0.0, 0.0, 0.0)
 		self.dest_rect = sdl3.SDL_FRect(0.0, 0.0, 0.0, 0.0)
 
-		if system == "Linux":
-			self.surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
-			self.context = cairo.Context(self.surf)
-			self.layout = PangoCairo.create_layout(self.context)
-
-		else:
-			self.cache: dict[
-				tuple[str, Win32Font | None, int, int, int, int, int, int, int, int],
-				list[sdl3.SDL_FRect | sdl3.LP_SDL_Texture],
-			] = {}
-			self.ca_li: list[tuple[str, Win32Font | None, int, int, int, int, int, int, int, int]] = []
-			self.y_offset_dict: dict[float, int] = {}
+		self.surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
+		self.context = cairo.Context(self.surf)
+		self.layout = PangoCairo.create_layout(self.context)
 
 		self.text_background_colour = ColourRGBA(0, 0, 0, 255)
 		self.pretty_rect: tuple[int, int, int, int] | None = None
 		self.real_bg: bool = False
 		self.alpha_bg: bool = False
 		self.force_gray: bool = False
-		self.f_dict: dict[float, Win32Font | tuple[str, int, float]] = {}
+		self.f_dict: dict[float, tuple[str, int, float]] = {}
 		self.ttc: dict[
 			tuple[int, str, int, int, int, int, int, int, int, int],
 			list[sdl3.SDL_FRect | sdl3.LP_SDL_Texture | int | bool],
@@ -467,32 +275,25 @@ class TDraw:
 		self.ttc.clear()
 		self.ttl.clear()
 
-	def win_prime_font(self, name: str, size: int, user_handle: float, weight: int, y_offset: int = 0) -> None:
-		self.f_dict[user_handle] = Win32Font(name, int(size * self.scale), weight)
-		self.y_offset_dict[user_handle] = y_offset
-
 	def prime_font(self, name: str, size: float, user_handle: float, offset: int = 0) -> None:
 		self.f_dict[user_handle] = (name + " " + str(size * self.scale), offset, size * self.scale)
 
 	def get_text_wh(self, text: str, font: int, max_x: int, wrap: bool = False) -> tuple[int, int] | None:
-		if system == "Linux":
-			self.layout.set_font_description(Pango.FontDescription(self.f_dict[font][0]))
-			self.layout.set_ellipsize(Pango.EllipsizeMode.END)
-			self.layout.set_width(max_x * 1000)
-			if wrap:
-				self.layout.set_height(20000 * 1000)
-			else:
-				self.layout.set_height(0)
+		self.layout.set_font_description(Pango.FontDescription(self.f_dict[font][0]))
+		self.layout.set_ellipsize(Pango.EllipsizeMode.END)
+		self.layout.set_width(max_x * 1000)
+		if wrap:
+			self.layout.set_height(20000 * 1000)
+		else:
+			self.layout.set_height(0)
 
-			try:
-				self.layout.set_text(text, -1)
-			except Exception:
-				logging.exception(f"Exception in get_text_wh for: {text}")
-				self.layout.set_text(text.encode("utf-8", "replace").decode("utf-8"), -1)
+		try:
+			self.layout.set_text(text, -1)
+		except Exception:
+			logging.exception(f"Exception in get_text_wh for: {text}")
+			self.layout.set_text(text.encode("utf-8", "replace").decode("utf-8"), -1)
 
-			return self.layout.get_pixel_size()
-		# return self.__win_text_xy(text, font)
-		return self.__win_text_xy(text, font, max_x, wrap)
+		return self.layout.get_pixel_size()
 
 	def get_y_offset(self, text: str, font: int, max_x: int, wrap: bool = False) -> int:
 		"""HACKY"""
@@ -716,7 +517,7 @@ class TDraw:
 		context.set_source_rgb(colour.r / 255, colour.g / 255, colour.b / 255)
 
 		if font not in self.f_dict:
-			logging.info("Font not loaded: " + str(font))
+			logging.info(f"Font not loaded: {font!s}")
 			return 10
 
 		# desc = Pango.FontDescription(self.f_dict[font][0])
@@ -790,138 +591,12 @@ class TDraw:
 			return dst.h
 		return dst.w
 
-	# WINDOWS --------------------------------------------------------
-
-	def __win_text_xy(self, text: str, font: int | None, max_x: int, wrap: bool) -> tuple[int, int] | None:
-		if font is None or font not in self.f_dict:
-			logging.info("Missing Font")
-			logging.info(font)
-			return None
-		return self.f_dict[font].get_metrics(text, max_x, wrap)
-
-	def __win_render_text(
-		self,
-		key: list[sdl3.SDL_FRect | sdl3.LP_SDL_Texture],
-		x: float,
-		y: float,
-		range_top: int,
-		range_height: int,
-		align: int,
-	) -> None:
-		sd = key
-
-		sd[0].x = round(x)
-		sd[0].y = round(y)
-		if align == 1:
-			sd[0].x = round(x) - sd[0].w
-		elif align == 2:
-			sd[0].x -= int(sd[0].w / 2)
-
-		if range_height is not None and range_height < sd[0].h - 20:
-			if range_top + range_height > sd[0].h:
-				# range_top = 0
-				range_height = sd[0].h - range_top
-
-			self.source_rect.y = sd[0].h - round(range_height) - round(range_top)
-			self.source_rect.w = sd[0].w
-			self.source_rect.h = round(range_height)
-
-			self.dest_rect.x = sd[0].x
-			self.dest_rect.y = sd[0].y
-			self.dest_rect.w = sd[0].w
-			self.dest_rect.h = round(range_height)
-
-			sdl3.SDL_RenderCopyEx(
-				self.renderer, sd[1], self.source_rect, self.dest_rect, 0, None, sdl3.SDL_FLIP_VERTICAL
-			)
-			return
-
-		sdl3.SDL_RenderCopyEx(self.renderer, sd[1], None, sd[0], 0, None, sdl3.SDL_FLIP_VERTICAL)
-
-	def __draw_text_windows(
-		self,
-		x: float,
-		y: float,
-		text: str,
-		bg: ColourRGBA,
-		fg: ColourRGBA,
-		font: Win32Font | None = None,
-		align: int = 0,
-		wrap: bool = False,
-		max_x: int = 100,
-		max_y: int | None = None,
-		range_top: int = 0,
-		range_height: int | None = None,
-	) -> int:
-		y += self.y_offset_dict[font]
-
-		key = (text, font, fg.r, fg.g, fg.b, fg.a, bg.r, bg.g, bg.b, max_x)
-
-		if key in self.cache:
-			sd = self.cache[key]
-
-			self.__win_render_text(sd, x, y, range_top, range_height, align)
-			if wrap:
-				return sd[0].h
-			return sd[0].w
-
-		if font is None or font not in self.f_dict:
-			logging.info("Missing Font")
-			logging.info(font)
-			return 0
-
-		# perf_timer.set()
-
-		f = self.f_dict[font]
-
-		w, h = f.get_metrics(text, max_x, wrap)
-		if max_y and max_y > h:
-			max_y = h
-
-		im, c_bits = f.renderText(text, bg, fg, wrap, max_x, max_y)
-
-		s_image = im
-		ke = sdl3.SDL_MapRGB(s_image.contents.format, bg.r, bg.g, bg.b)
-		sdl3.SDL_SetColorKey(s_image, True, ke)
-		c = sdl3.SDL_CreateTextureFromSurface(self.renderer, s_image)
-		tex_w = c_float(0.0)
-		tex_h = c_float(0.0)
-		sdl3.SDL_GetTextureSize(c, byref(tex_w), byref(tex_h))
-		dst = sdl3.SDL_FRect(round(x), round(y))
-		dst.w = int(tex_w.value)
-		dst.h = int(tex_h.value)
-
-		sdl3.SDL_DestroySurface(s_image)
-		# im.close()
-
-		if align == 1:
-			dst.x = round(x) - dst.w
-
-		elif align == 2:
-			dst.x -= int(dst.w / 2)
-
-		# sdl3.SDL_RenderCopy(renderer, c, None, dst)
-		# sdl3.SDL_RenderCopyEx(self.renderer, c, None, dst, 0, None, sdl3.SDL_FLIP_VERTICAL)
-
-		# logging.info(perf_timer.get())
-		self.cache[key] = [dst, c]
-		self.__win_render_text([dst, c], x, y, range_top, range_height, align)
-
-		self.ca_li.append(key)
-
-		if len(self.ca_li) > 350:
-			sdl3.SDL_DestroyTexture(self.cache[self.ca_li[0]][1])
-			del self.cache[self.ca_li[0]]
-			del self.ca_li[0]
-
-		return dst.w
-
 	def text(
 		self,
 		location: list[int],
 		text: str,
 		colour: ColourRGBA,
-		font: int | Win32Font,
+		font: int,
 		max_w: int = 4000,
 		bg: ColourRGBA | None = None,
 		range_top: int = 0,
@@ -952,34 +627,17 @@ class TDraw:
 				if len(location) > 4:
 					max_h = location[4]
 
-				if system == "Linux":
-					return self.__draw_text_cairo(
-						location,
-						text,
-						colour,
-						font,
-						location[3],
-						bg,
-						max_y=max_h,
-						wrap=True,
-						range_top=range_top,
-						range_height=range_height,
-					)
-				return self.__draw_text_windows(
-					location[0],
-					location[1],
+				return self.__draw_text_cairo(
+					location,
 					text,
-					bg,
 					colour,
 					font,
-					0,
-					True,
 					location[3],
+					bg,
 					max_y=max_h,
+					wrap=True,
 					range_top=range_top,
 					range_height=range_height,
 				)
 
-		if system == "Linux":
-			return self.__draw_text_cairo(location, text, colour, font, max_w, bg, align, real_bg=real_bg, key=key)
-		return self.__draw_text_windows(location[0], location[1], text, bg, colour, font, align=align, max_x=max_w)
+		return self.__draw_text_cairo(location, text, colour, font, max_w, bg, align, real_bg=real_bg, key=key)
