@@ -35,6 +35,7 @@ import threading
 import time
 import urllib.parse
 import zipfile
+from collections import deque
 from dataclasses import dataclass, field
 from pathlib import Path
 from typing import TYPE_CHECKING
@@ -133,10 +134,10 @@ class TauonPlaylist:
 		4 hide_title on playlist folders (bool)
 		5 selected (int)
 		6 uuid_int (int)
-		7 last_folder import path (string)
+		7 last_folder import paths (list[str])
 		8 hidden (bool)
 		9 locked (bool)
-		10 parent_playlist_id <- Filter (string)
+		10 parent_playlist_id <- Filter (int)
 		11 persist_time_positioning
 	"""
 
@@ -149,10 +150,10 @@ class TauonPlaylist:
 	uuid_int: int
 	last_folder: list[
 		str
-	]  # last folder import path (string) - TODO(Martin): BUG - we are using this both as string and list of strings in various parts of code
+	]  # last folder import paths
 	hidden: bool
 	locked: bool
-	parent_playlist_id: str  # Filter parent playlist id (string)
+	parent_playlist_id: int  # Filter parent playlist id
 	persist_time_positioning: bool  # Persist time positioning
 	playlist_file: str = ""  # Playlist may be automatically loaded to and from this filepath
 	auto_export: bool = False
@@ -170,8 +171,8 @@ class Timer:
 	"""A seconds based timer"""
 
 	def __init__(self, force: float | None = None) -> None:
-		self.start = 0
-		self.end = 0
+		self.start: float = 0
+		self.end: float = 0
 		self.set()
 		if force:
 			self.force_set(force)
@@ -470,7 +471,7 @@ def point_proximity_test(a: list[int], b: list[int], p: float) -> bool:
 
 def point_distance(a: list[int], b: list[int]) -> float:
 	"""Get distance between two points"""
-	return math.sqrt(abs(a[0] - b[0]) ** 2 + abs(b[1] - b[1]) ** 2)
+	return math.sqrt(abs(a[0] - b[0]) ** 2 + abs(a[1] - b[1]) ** 2)
 
 
 def rm_16(line: str) -> str:
@@ -573,7 +574,7 @@ def get_modify_date_string(modify_date: float) -> str:
 				else:
 					try:
 						modify_date = time.localtime(modify_date)
-					except:
+					except Exception:
 						ago_str = _("Unknown")
 					else:
 						if ago < 365:
@@ -764,7 +765,7 @@ def is_light(colour: ColourRGBA) -> bool:
 	return test_lumi(colour) < 0.2
 
 
-def folder_file_scan(path: str, extensions: str) -> float:
+def folder_file_scan(path: str, extensions: set[str]) -> float:
 	match = 0
 	count = sum([len(files) for r, d, files in os.walk(path)])
 	for ext in extensions:
@@ -818,7 +819,7 @@ def is_music_related(string: str) -> bool:
 	return False
 
 
-def archive_file_scan(path: str, extensions: str, launch_prefix: str = "") -> float:
+def archive_file_scan(path: str, extensions: set[str], launch_prefix: str = "") -> float:
 	"""Get ratio of given file extensions in archive"""
 	ext = os.path.splitext(path)[1][1:].lower()
 	# logging.info(path)
@@ -893,33 +894,33 @@ def archive_file_scan(path: str, extensions: str, launch_prefix: str = "") -> fl
 				return 0
 
 		elif ext == "zip":
-			zip_ref = zipfile.ZipFile(path, "r")
-			matches = 0
-			count = 0
-			# logging.info(zip_ref.namelist())
-			for fi in zip_ref.namelist():
-				for ty in extensions:
-					if fi[len(ty) * -1 :].lower() == ty:
-						matches += 1
-						break
-					if is_ignorable_file(fi):
-						count -= 1
-						break
-					if is_music_related(fi):
-						matches += 5
-				count += 1
-			if count == 0:
-				# logging.info("Archive has no files")
-				# logging.info("   --- " + path)
-				return 0
-			if count > 300:
-				# logging.info("Zip archive has many files")
-				# logging.info("   --- " + path)
-				return 0
-			if matches == 0:
-				# logging.info("Zip archive does not appear to contain audio files")
-				# logging.info("   --- " + path)
-				return 0
+			with zipfile.ZipFile(path, "r") as zip_ref:
+				matches = 0
+				count = 0
+				# logging.info(zip_ref.namelist())
+				for fi in zip_ref.namelist():
+					for ty in extensions:
+						if fi[len(ty) * -1 :].lower() == ty:
+							matches += 1
+							break
+						if is_ignorable_file(fi):
+							count -= 1
+							break
+						if is_music_related(fi):
+							matches += 5
+					count += 1
+				if count == 0:
+					# logging.info("Archive has no files")
+					# logging.info("   --- " + path)
+					return 0
+				if count > 300:
+					# logging.info("Zip archive has many files")
+					# logging.info("   --- " + path)
+					return 0
+				if matches == 0:
+					# logging.info("Zip archive does not appear to contain audio files")
+					# logging.info("   --- " + path)
+					return 0
 		else:
 			return 0
 
@@ -972,7 +973,9 @@ def filename_to_metadata(filename: str) -> tuple[str, str]:
 
 def get_artist_strip_feat(track_object: TrackClass) -> str:
 	artist_name = track_object.artist  # .lower()
-	if track_object.album_artist:
+	if track_object.misc.get("artists"):
+		artist_name = track_object.misc["artists"][0]
+	elif track_object.album_artist:
 		if (
 			"feat." in artist_name
 			or "pres." in artist_name
@@ -1172,7 +1175,7 @@ def sleep_timeout(condition_function: Callable[[], bool], time_limit: int = 2) -
 		return
 	timer = Timer()
 	timer.set()
-	while condition_function():
+	while not condition_function():
 		time.sleep(0.01)
 		if timer.get() > time_limit:
 			break
@@ -1186,3 +1189,75 @@ def tryint(string: str) -> int | str:
 	except Exception:
 		logging.exception("Unknown error trying to convert string to int!")
 		return string
+
+
+class FPSCounter:
+	"""A robust FPS counter that handles pauses and provides smooth measurements.
+
+	Args:
+		window_size: Number of frames to average over (default: 60)
+		min_update_interval: Minimum time between updates in seconds (default: 0.1)
+		max_frame_time: Maximum expected frame time in seconds. If exceeded,
+						frame is considered anomalous (default: 1.0)
+
+	"""
+
+	def __init__(self, window_size: int = 60, min_update_interval: float = 0.1, max_frame_time: float = 1.0) -> None:
+		self.window_size = window_size
+		self.min_update_interval = min_update_interval
+		self.max_frame_time = max_frame_time
+
+		self.frame_times = deque(maxlen=window_size)
+		self.last_tick_time: float | None = None
+		self.last_update_time: float | None = None
+		self.cached_fps: float = 0.0
+		self.frame_count: int = 0
+
+	def tick(self) -> None:
+		"""Call this once per frame to record the frame timing."""
+		current_time = time.perf_counter()
+
+		if self.last_tick_time is not None:
+			frame_time = current_time - self.last_tick_time
+
+			# Ignore anomalous frame times (e.g., after a pause)
+			if frame_time <= self.max_frame_time:
+				self.frame_times.append(frame_time)
+
+		self.last_tick_time = current_time
+		self.frame_count += 1
+
+	def get(self) -> float:
+		"""Get the current FPS value.
+
+		Returns the cached value if called within min_update_interval,
+		otherwise recalculates based on recent frame times.
+
+		Returns:
+			float: Current FPS value, or 0.0 if insufficient data
+
+		"""
+		current_time = time.perf_counter()
+
+		# Return cached value if we updated recently
+		if (self.last_update_time is not None and
+				current_time - self.last_update_time < self.min_update_interval):
+			return self.cached_fps
+
+		# Calculate new FPS value
+		if len(self.frame_times) == 0:
+			self.cached_fps = 0.0
+		else:
+			avg_frame_time = sum(self.frame_times) / len(self.frame_times)
+			self.cached_fps = 1.0 / avg_frame_time if avg_frame_time > 0 else 0.0
+
+		self.last_update_time = current_time
+		return self.cached_fps
+
+	def reset(self) -> None:
+		"""Reset the FPS counter."""
+		self.frame_times.clear()
+		self.last_tick_time = None
+		self.last_update_time = None
+		self.cached_fps = 0.0
+		self.frame_count = 0
