@@ -46,22 +46,27 @@ from tauon.t_modules.t_enums import PlayerState, PlayingState
 from tauon.t_modules.t_extra import Timer, shooter, tmp_cache_dir
 
 if TYPE_CHECKING:
-	from tauon.t_modules.t_main import PlayerCtl, Tauon, TrackClass
+	from ctypes import CDLL
+	from subprocess import Popen
+
+	from tauon.t_modules.t_main import GuiVar, PlayerCtl, Tauon, TrackClass
+	from tauon.t_modules.t_prefs import Prefs
+	from tauon.t_modules.t_spot import SpotCtl
 
 
 class LibreSpot:
 	def __init__(self, tauon: Tauon) -> None:
 		# fmt:off
-		self.tauon           = tauon
-		self.aud             = tauon.aud
-		self.gui             = tauon.gui
-		self.msys            = tauon.msys
-		self.pctl            = tauon.pctl
-		self.prefs           = tauon.prefs
-		self.spot_ctl        = tauon.spot_ctl
-		self.librespot_p     = tauon.librespot_p
+		self.tauon:      Tauon = tauon
+		self.aud:         CDLL = tauon.aud
+		self.gui:       GuiVar = tauon.gui
+		self.windows:        bool = tauon.windows
+		self.pctl:   PlayerCtl = tauon.pctl
+		self.prefs:      Prefs = tauon.prefs
+		self.spot_ctl: SpotCtl = tauon.spot_ctl
+		self.librespot_p: Popen[bytes] | None = tauon.librespot_p
 		self.show_message    = tauon.show_message
-		self.cache_directory = tauon.cache_directory
+		self.cache_directory: Path = tauon.cache_directory
 		self.running:   bool = False
 		self.flush:     bool = False
 		# fmt:on
@@ -140,7 +145,7 @@ class LibreSpot:
 		self.running = False
 		self.pctl.spot_playing = False
 		if self.librespot_p:
-			if self.msys:
+			if self.windows:
 				self.librespot_p.terminate()
 				# self.librespot_p.communicate()
 			else:
@@ -548,7 +553,7 @@ def find_library(libname: str) -> Path | None:
 def get_phazor_path(pctl: PlayerCtl) -> Path:
 	"""Locate the PHaZOR library in the specified priority order.
 
-	Tries .so, .dll, .pyd, .dynlib in that order and finally uses find_library as a fallback.
+	Tries .so, .dll, .pyd, .dylib in that order and finally uses find_library as a fallback.
 
 	:param pctl: PlayerCtl object containing installation details
 	:return: Path to the library file
@@ -558,9 +563,9 @@ def get_phazor_path(pctl: PlayerCtl) -> Path:
 	base_path = Path(pctl.install_directory).parent.parent / "build"
 
 	# Define the library name and extensions in priority order
-	lib_name = "phazor-pw" if pctl.prefs.pipewire else "phazor"
+	lib_name = "phazor-pw" if (not pctl.windows and not pctl.tauon.macos) and pctl.prefs.pipewire else "phazor"
 
-	extensions = [".so", ".dll", ".pyd", ".dynlib"]
+	extensions = [".so", ".dll", ".pyd", ".dylib"]
 
 	# Check explicitly for each file
 	for ext in extensions:
@@ -636,13 +641,13 @@ def player4(tauon: Tauon) -> None:
 					if tp is not None:
 						p = tp
 
-		logging.debug("Detected Replay gain")
+		logging.debug("Detected ReplayGain")
 		logging.debug("GAIN: " + str(g))
 		logging.debug("PEAK: " + str(p))
 		logging.debug("FINAL: " + str(min(10 ** ((g + prefs.replay_preamp) / 20), 1 / p)))
 
 		if p == 0:
-			logging.warning("Detected replay gain peak of 0")
+			logging.warning("Detected ReplayGain peak of 0")
 			return 1
 		pctl.active_replaygain = g
 		return min(10 ** ((g + prefs.replay_preamp) / 20), 1 / p)
@@ -854,6 +859,9 @@ def player4(tauon: Tauon) -> None:
 			gui.level_update = True
 
 		if chrome_mode:
+			if tauon.chrome is None:
+				logging.critical("This should not happen, tauon.chrome was None")
+				return
 			if pctl.playerCommandReady:
 				command = pctl.playerCommand
 				# logging.info(command)
@@ -988,6 +996,9 @@ def player4(tauon: Tauon) -> None:
 				stall_timer.set()
 				pctl.download_time = 0
 				target_object = pctl.target_object
+				if not target_object:
+					logging.exception("This shouldn't happen, target_object was None")
+					continue
 				target_path = target_object.fullpath
 				subtrack = target_object.subtrack
 				aud.set_subtrack(subtrack)
@@ -1036,7 +1047,7 @@ def player4(tauon: Tauon) -> None:
 							)
 							# tauon.spot_ctl.preparing_spotify = False
 							if pctl.playerCommand == "spotcon":  # such spaghetti code
-								state = 4
+								tauon.player4_state = PlayerState.SPOTIFY_MODE
 								pctl.playerCommand = ""
 								pctl.playerCommandReady = False
 
@@ -1146,9 +1157,9 @@ def player4(tauon: Tauon) -> None:
 
 					# TODO(Martin): The GUI logger does not support multiline
 					# logging.info(f"{loaded_track.title} -> {target_object.title}\nlength: {length!s}\nposition: {position!s}\nWe are {remain!s} from end")
-					logging.info(loaded_track.title + " -> " + target_object.title)
-					logging.info(" --- length: " + str(length))
-					logging.info(" --- position: " + str(position))
+					logging.info(f"{loaded_track.title} -> {target_object.title}")
+					logging.info(f" --- length: {length!s}")
+					logging.info(f" --- position: {position!s}")
 					logging.info(f" --- We are {remain!s} from end")
 
 					if loaded_track.is_network or length == 0:
