@@ -37,7 +37,7 @@ if TYPE_CHECKING:
 	from tekore._client.full import Spotify
 	from tekore._model.album.full import FullAlbum
 
-	from tauon.t_modules.t_main import Strings, Tauon, TrackClass
+	from tauon.t_modules.t_main import Tauon, TrackClass
 
 tekore_imported = False
 try:
@@ -152,7 +152,6 @@ class SpotCtl:
 					self.token = tk.refresh_pkce_token(self.tauon.prefs.spot_client, self.tauon.prefs.spotify_token)
 			except Exception:
 				logging.exception("FAILED TO LOAD TOKEN")
-				raise
 				self.show_message(_("Please re-authenticate Spotify in settings"))
 				self.tauon.prefs.spotify_token = ""
 		else:
@@ -463,7 +462,7 @@ class SpotCtl:
 
 		self.preparing_spotify = False
 
-	def play_target(self, id: int, force_new_device: bool = False, start_time: int | None = 0, start_callback=None):
+	def play_target(self, id: int | str, force_new_device: bool = False, start_time: int | None = 0, start_callback=None) -> None:
 		start_time = None if not start_time else int(start_time * 1000)
 
 		self.coasting = False
@@ -475,7 +474,7 @@ class SpotCtl:
 			)
 			return
 
-		logging.info("Want play spotify target " + str(id))
+		logging.info(f"Want play spotify target {id!s}")
 		# Sort devices
 		start_new_device = False
 		done = False
@@ -571,15 +570,21 @@ class SpotCtl:
 						start_callback()
 
 					# self.tauon.pctl.playing_state = 3
-				elif self.tauon.msys:
-					self.launching_spotify = True
-					p = os.getenv("APPDATA") + "\\Spotify\\Spotify.exe"
-					if not os.path.isfile(p):
-						self.launching_spotify = False
-						self.preparing_spotify = False
-						logging.info("Spotify app exe not found, aborting.")
-						return
-					subprocess.Popen([p])
+					elif self.tauon.windows:
+						self.launching_spotify = True
+						appdata = os.getenv("APPDATA")
+						if not appdata:
+							self.launching_spotify = False
+							self.preparing_spotify = False
+							logging.info("APPDATA is not set, cannot locate Spotify app.")
+							return
+						p = appdata + "\\Spotify\\Spotify.exe"
+						if not os.path.isfile(p):
+							self.launching_spotify = False
+							self.preparing_spotify = False
+							logging.info("Spotify app exe not found, aborting.")
+							return
+						subprocess.Popen([p])
 					logging.info("Launching spotify app exe")
 					time.sleep(3)
 				else:
@@ -802,7 +807,7 @@ class SpotCtl:
 		pages = self.spotify.all_pages(p.tracks)
 		for page in pages:
 			for item in page.items:
-				if type(item.track) == tk.model.FullPlaylistTrack:
+				if type(item.track) is tk.model.FullPlaylistTrack:
 					nt = self.load_track(item.track, include_album_url=True)
 					self.tauon.pctl.master_library[nt.index] = nt
 					playlist.append(nt.index)
@@ -819,6 +824,7 @@ class SpotCtl:
 		self.tauon.pctl.gen_codes[self.tauon.pl_to_id(len(self.tauon.pctl.multi_playlist) - 1)] = f'spl"{id}"'
 		if not silent:
 			self.tauon.switch_playlist(len(self.tauon.pctl.multi_playlist) - 1)
+		return None
 
 	# def rec_playlist(self, artist_url: str, track_url: str) -> None:
 	#
@@ -921,7 +927,7 @@ class SpotCtl:
 			logging.exception("Spotify upload error!")
 			self.show_message(_("Spotify upload error!"), mode="error")
 
-	def load_album(self, album: FullAlbum, playlist: list[int] | None):
+	def load_album(self, album: FullAlbum, playlist: list[int] | None) -> None:
 		# a = item
 		album_url = album.external_urls["spotify"]
 		art_url = album.images[0].url
@@ -972,11 +978,10 @@ class SpotCtl:
 			playlist.append(nt.index)
 
 	def load_track(
-		self, track: TrackClass, update_master_count: bool = True, include_album_url: bool = False
+		self, track: tk.model.FullPlaylistTrack, update_master_count: bool = True, include_album_url: bool = False
 	) -> TrackClass:
 		if "spotify" in track.external_urls:
 			pr = self.current_imports.get(track.external_urls["spotify"])
-
 		else:
 			pr = False
 
