@@ -32,6 +32,7 @@ import urllib.request
 
 import mutagen
 
+from tauon.t_modules.t_enums import Backend
 from tauon.t_modules.t_extra import filename_safe
 
 if sys.platform != "win32":
@@ -119,6 +120,7 @@ class StreamEnc:
 		retry = 5
 		while True:
 			try:
+				urllib.request.http.client.HTTPResponse._read_status = NiceToICY
 				r = urllib.request.Request(self.url)
 				# r.add_header('GET', '1')
 				if not self.url.endswith(".ts"):
@@ -139,6 +141,8 @@ class StreamEnc:
 				logging.error("Connection failed")
 				self.tauon.show_message(_("Failed to establish a connection"), str(e), mode="error")
 				return False
+			finally:
+				urllib.request.http.client.HTTPResponse._read_status = ORIGINAL_HTTP_CLIENT_READ_STATUS
 			break
 
 		self.download_process = threading.Thread(target=self.run_download, args=([r]))
@@ -149,7 +153,7 @@ class StreamEnc:
 
 	def pump(self) -> None:
 		aud = self.tauon.aud
-		if self.tauon.prefs.backend != 4 or not aud:
+		if self.tauon.prefs.backend != Backend.PHAZOR or not aud:
 			logging.error("Radio error: Phazor not loaded")
 			return
 		self.pump_running = True
@@ -169,7 +173,7 @@ class StreamEnc:
 		# fmt:on
 
 		startupinfo = None
-		if self.tauon.msys:
+		if self.tauon.windows:
 			startupinfo = subprocess.STARTUPINFO()
 			startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
 		decoder = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, startupinfo=startupinfo)
@@ -239,6 +243,12 @@ class StreamEnc:
 		self.encode_running = True
 
 		try:
+			ffmpeg_path = self.tauon.get_ffmpeg()
+			if ffmpeg_path is None:
+				logging.error("FFMPEG could not be found for stream encoder")
+				self.encode_running = False
+				return
+
 			while self.c < 20:
 				if self.abort:
 					self.encode_running = False
@@ -259,12 +269,13 @@ class StreamEnc:
 				rate = "44100"
 
 			target_file = self.tauon.cache_directory / "stream" / ext
+			target_file.parent.mkdir(parents=True, exist_ok=True)
 			if target_file.is_file():
 				target_file.unlink()
 
 			# fmt:off
 			cmd = [
-				str(self.tauon.get_ffmpeg()),
+				str(ffmpeg_path),
 				"-loglevel", "quiet",
 				"-i", "pipe:0",
 				"-acodec", "pcm_s16le",
@@ -286,7 +297,7 @@ class StreamEnc:
 			##cmd = ["opusenc", "--raw", "--raw-rate", "48000", "-", target_file]
 			# fmt:off
 			cmd = [
-				"ffmpeg",
+				str(ffmpeg_path),
 				"-loglevel", "quiet",
 				"-f", "s16le",
 				"-ar", rate,
@@ -297,7 +308,7 @@ class StreamEnc:
 			# fmt:on
 			encoder = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
 
-			def save_track():
+			def save_track() -> None:
 				# self.tauon.recorded_songs.append(song)
 
 				save_file = f"{datetime.datetime.now():%Y-%m-%d %H-%M-%S} - "
@@ -474,6 +485,12 @@ class StreamEnc:
 					logging.info("Abort stream connection")
 					self.download_running = False
 					return
+				if not chunk:
+					r.close()
+					logging.info("Stream connection closed")
+					self.download_running = False
+					self.abort = True
+					return
 
 				if chunk:
 					if not icy or m_remain > len(chunk):
