@@ -1,6 +1,8 @@
 """Tauon Music Box - Tag Module
 
 The purpose of this module is to read metadata from FLAC, OGG, OPUS, APE and WV files
+
+There is additional parsing logic in in Tauon.tag_scan()
 """
 
 # Copyright © 2015-2019, Taiko2k captain(dot)gxj(at)gmail.com
@@ -108,6 +110,46 @@ def parse_picture_block(f: BufferedReader) -> bytes:
 	return f.read(a)
 
 
+def parse_wavpack_header(header_data: bytes) -> tuple[int, float] | None:
+	"""Parse a WavPack block header and return (sample_rate, length_seconds)."""
+	if len(header_data) < 32:
+		return None
+
+	try:
+		header = struct.unpack("<4cIH2B5I", header_data[:32])
+	except struct.error:
+		return None
+
+	sample_rates = [
+		6000,
+		8000,
+		9600,
+		11025,
+		12000,
+		16000,
+		22050,
+		24000,
+		32000,
+		44100,
+		48000,
+		64000,
+		88200,
+		96000,
+		192000,
+	]  # Adapted from example in WavPack/cli/wvparser.c
+
+	sample_rate_index = (header[11] & (15 << 23)) >> 23
+	if sample_rate_index >= len(sample_rates):
+		return None
+
+	sample_rate = sample_rates[sample_rate_index]
+	if sample_rate <= 0:
+		return None
+
+	length = header[8] / sample_rate
+	return sample_rate, length
+
+
 class TrackFile:
 	"""Base class for codec classes"""
 
@@ -116,22 +158,23 @@ class TrackFile:
 		self.file: BufferedReader | None = None
 		self.has_picture = False # Wav does not need this
 
-		self.picture      = "" # Wav does not need this
-		self.filepath     = ""
-		self.album_artist = ""
-		self.artist       = ""
-		self.genre        = ""
-		self.date         = "" # Wav does not need this
-		self.comment      = "" # Wav does not need this
-		self.album        = ""
-		self.track_number = ""
-		self.track_total  = "" # Wav does not need this
-		self.title        = ""
-		self.encoder      = "" # Wav does not need this
-		self.disc_number  = "" # Wav does not need this
-		self.disc_total   = "" # Wav does not need this
-		self.lyrics       = "" # Wav does not need this
-		self.composer     = "" # Wav does not need this
+		self.picture       = "" # Wav does not need this
+		self.filepath      = ""
+		self.album_artist  = ""
+		self.artist        = ""
+		self.genre         = ""
+		self.date          = "" # Wav does not need this
+		self.comment       = "" # Wav does not need this
+		self.album         = ""
+		self.track_number  = ""
+		self.track_total   = "" # Wav does not need this
+		self.title         = ""
+		self.encoder       = "" # Wav does not need this
+		self.disc_number   = "" # Wav does not need this
+		self.disc_total    = "" # Wav does not need this
+		self.lyrics        = "" # Wav does not need this
+		self.synced_lyrics = ""
+		self.composer      = "" # Wav does not need this
 		self.misc: dict[str, str | float | list[str]] = {} # Wav does not need this
 
 		self.sample_rate = 48000 # OPUS files are always 48000
@@ -244,7 +287,9 @@ class Flac(TrackFile):
 					elif a == "metadata_block_picture":
 						logging.info("Tag Scanner: Found picture inside vorbis comment inside a FLAC file. Ignoring")
 						logging.info(f"      In file: {self.filepath}")
-					elif a in ("lyrics", "unsyncedlyrics"):
+					elif a == "lyrics":
+						self.synced_lyrics = b.decode("utf-8")
+					elif a == "unsyncedlyrics":
 						self.lyrics = b.decode("utf-8")
 					elif a == "replaygain_track_gain":
 						self.misc["replaygain_track_gain"] = float(
@@ -383,7 +428,7 @@ class Flac(TrackFile):
 class Opus(TrackFile):
 	def __init__(self, file: str) -> None:
 		super().__init__()
-		self.filepath = file
+		self.filepath: str = file
 		self.length = 0
 
 	def get_more(self, f: BufferedReader, v: BytesIO) -> int:
@@ -535,7 +580,9 @@ class Opus(TrackFile):
 						self.disc_number = b.decode("utf-8")
 					elif a in ("disctotal", "totaldiscs"):
 						self.disc_total = b.decode("utf-8")
-					elif a in ("lyrics", "unsyncedlyrics"):
+					elif a == "lyrics":
+						self.synced_lyrics = b.decode("utf-8")
+					elif a == "unsyncedlyrics":
 						self.lyrics = b.decode("utf-8")
 					elif a == "composer":
 						self.composer = b.decode("utf-8")
@@ -768,17 +815,35 @@ class Ape(TrackFile):
 
 		elif ".tta" in self.filepath:
 			a.seek(0)
-			header = struct.unpack("<4c3H3L", a.read(22))
+			signature = a.read(4)
+
+			if signature == b"TTA1":
+				header = a.read(18)
+				if len(header) != 18:
+					logging.info("WARNING: TTA1 header is incomplete")
+				else:
+					_audio_format, _channels, bps, sample_rate, total_samples, _crc32 = struct.unpack("<3H3L", header)
+					if sample_rate > 0:
+						self.sample_rate = sample_rate
+					self.bit_depth = bps
+					if self.sample_rate > 0:
+						self.length = total_samples / self.sample_rate
+
+			elif signature == b"TTA2":
+				# TTA2 header:
+				# channels(2), bits_per_sample(2), sample_rate(4), channel_mask(4),
+				# total_samples(8), data_size(8)
+				header = a.read(28)
+				if len(header) != 28:
+					logging.info("WARNING: TTA2 header is incomplete")
+				else:
+					_channels, bps, sample_rate, _channel_mask, total_samples, _data_size = struct.unpack("<HHIIQQ", header)
+					if sample_rate > 0:
+						self.sample_rate = sample_rate
+					self.bit_depth = bps
+					if self.sample_rate > 0 and total_samples > 0:
+						self.length = total_samples / self.sample_rate
 
-			if b"".join(header[0:3]) != b"TTA1":
-				self.sample_rate = header[7]
-				bps = header[6]
-				self.bit_depth = bps
-				# channels = header[5]
-				self.length = header[8] / self.sample_rate
-			elif b"".join(header[0:3]) != b"TTA2":
-				logging.info("WARNING: TTA2 type TTA file not supported")
-				# To do
 			else:
 				logging.info("WARNING: Does not appear to be a valid TTA file")
 		elif ".wv" in self.filepath:
@@ -787,37 +852,42 @@ class Ape(TrackFile):
 
 			a.seek(0)
 
-			#  I found that some WavPack files have padding at the beginning
-			#  So here I crudely search for the actual start
-			off = 0
-			while off < file_size - 100:
-				if a.read(4) == b"wvpk":
-					a.seek(-4, 1)
-					b = a.read(32)
-					header = struct.unpack("<4cIH2B5I", b)
+			found_wvpk = False
+			chunk_size = 64 * 1024
+			scan_limit = max(file_size - 31, 0)  # Need 32 bytes remaining to parse a full header
+			scan_pos = 0
 
-					sample_rates = [
-						6000,
-						8000,
-						9600,
-						11025,
-						12000,
-						16000,
-						22050,
-						24000,
-						32000,
-						44100,
-						48000,
-						64000,
-						88200,
-						96000,
-						192000,
-					]  # Adapted from example in WavPack/cli/wvparser.c
-					n = (header[11] & (15 << 23)) >> 23  # Does my head in this
-					self.sample_rate = sample_rates[n]
-					self.length = header[8] / self.sample_rate
+			while scan_pos < scan_limit:
+				a.seek(scan_pos)
+				read_size = min(chunk_size, scan_limit - scan_pos)
+				chunk = a.read(read_size)
+				if not chunk:
 					break
-			else:
+
+				search_from = 0
+				while True:
+					relative_pos = chunk.find(b"wvpk", search_from)
+					if relative_pos == -1:
+						break
+
+					signature_pos = scan_pos + relative_pos
+					a.seek(signature_pos)
+					parsed = parse_wavpack_header(a.read(32))
+					if parsed:
+						self.sample_rate, self.length = parsed
+						found_wvpk = True
+						break
+					search_from = relative_pos + 1
+
+				if found_wvpk:
+					break
+
+				# Step by chunk minus 3 bytes so signatures split across chunk boundaries are still visible.
+				if read_size <= 3:
+					break
+				scan_pos += read_size - 3
+
+			if not found_wvpk:
 				logging.info("Tag Scanner: Cannot verify WavPack file")
 		else:
 			logging.info("Tag Scanner: Does not appear to be an APE file")
@@ -842,7 +912,8 @@ class Wav(TrackFile):
 			remain = int.from_bytes(f.read(4), "little")
 
 			if wav_type != b"LIST":
-				f.seek(remain, io.SEEK_CUR)
+				# RIFF chunks are word-aligned, so odd-sized chunks include a pad byte.
+				f.seek(remain + (remain % 2), io.SEEK_CUR)
 			else:
 				info = f.read(4)
 				if info == b"INFO":
@@ -867,6 +938,13 @@ class Wav(TrackFile):
 							f.read(1)
 
 						remain -= 8 + size
+				else:
+					# Skip non-INFO LIST chunks (e.g. adtl) to avoid desyncing the chunk parser.
+					remaining_list_bytes = max(remain - 4, 0)
+					if remaining_list_bytes:
+						f.seek(remaining_list_bytes, io.SEEK_CUR)
+					if remain % 2 == 1:
+						f.seek(1, io.SEEK_CUR)
 
 		with wave.open(self.filepath, "rb") as wav:
 			self.sample_rate = wav.getframerate()
@@ -1073,8 +1151,7 @@ class M4a(TrackFile):
 			if name == b"":
 				return False
 
-			# logging.info("NAME: ", end="")
-			# logging.info(tail + b"." + name)
+			#logging.info(f"NAME: {tail}{b'.'}{name}")
 
 			# Too lazy to parse each sub atom, lets just grab the data out the sub atom and
 			# hope the file is formatted normally
@@ -1128,13 +1205,11 @@ class M4a(TrackFile):
 				self.disc_number = int(meta_get(f, size)[3])
 
 			# if tail[-4:] == b"----":
-			#
-			#
-			# 	if name == b'name':
+			# 	s_name: bytes = b""
+			# 	if name == b"name":
 			# 		s_name = f.read(size - 8)
 			# 		f.seek((size - 8) * -1, 1)
-			#
-			# 	elif name == b'data' and s_name != b"":
+			# 	elif name == b"data" and s_name != b"":
 			# 		data = f.read(size - 8)
 			# 		f.seek((size - 8) * -1, 1)
 			# 		logging.info(s_name)
