@@ -30,7 +30,7 @@ from http.server import BaseHTTPRequestHandler, HTTPServer
 from socketserver import ThreadingMixIn
 from typing import TYPE_CHECKING
 
-from tauon.t_modules.t_enums import PlayingState, StopMode
+from tauon.t_modules.t_enums import Backend, PlayingState, StopMode
 from tauon.t_modules.t_extra import Timer
 
 if TYPE_CHECKING:
@@ -78,7 +78,7 @@ class VorbisMonitor:
 		ogg = b.read(4)
 
 		if ogg != b"OggS":
-			f = data.find(b"Oggs")
+			f = data.find(b"OggS")
 			self.reset(self.tries)
 			if f > -1:
 				logging.info("Ogg stream synced")
@@ -97,7 +97,14 @@ class VorbisMonitor:
 
 		b.seek(0, io.SEEK_SET)
 		header = struct.unpack("<4sBBqIIiB", b.read(27))
-		segs = struct.unpack("B" * header[7], b.read(header[7]))
+		segment_count = header[7]
+
+		# Ensure we can read the segment table
+		seg_bytes = b.read(segment_count)
+		if len(seg_bytes) < segment_count:
+			return  # wait for more data
+
+		segs = seg_bytes
 
 		length = 0
 		for s in segs:
@@ -155,40 +162,82 @@ class VorbisMonitor:
 def send_file(path: str, mime: str, server) -> None:
 	range_req = False
 	start = 0
-	end = 0
+	end = None
+	range_header = server.headers.get("Range")
 
-	if "Range" in server.headers:
-		range_req = True
-		b = server.headers["Range"].split("=")[1]
-		start, end = b.split("-")
-		start = int(start)
+	try:
+		with open(path, "rb") as f:
+			f.seek(0, 2)
+			length = f.tell()
 
-	with open(path, "rb") as f:
-		f.seek(0, 2)
-		length = f.tell()
-		f.seek(start, 0)
+			if range_header:
+				try:
+					unit, value = range_header.split("=", 1)
+					if unit.strip().lower() != "bytes" or "," in value:
+						raise ValueError
+					start_str, end_str = value.split("-", 1)
+					if not start_str and not end_str:
+						raise ValueError
+					if start_str:
+						start = int(start_str)
+						if start < 0:
+							raise ValueError
+						end = int(end_str) if end_str else length - 1
+						if end < 0:
+							raise ValueError
+					else:
+						# Suffix ranges: bytes=-500 means last 500 bytes
+						suffix_length = int(end_str)
+						if suffix_length <= 0:
+							raise ValueError
+						start = max(0, length - suffix_length)
+						end = length - 1
+					range_req = True
+				except (IndexError, ValueError):
+					range_req = False
+					start = 0
+					end = None
 
-		if range_req:
-			server.send_response(206)
-			server.send_header("Content-Range", f"bytes {start}-{length - 1}/{length}")
-			server.send_header("Content-Length", str(length))
-			server.send_header("Content-Type", mime)
-			f.seek(start)
+			if range_req:
+				if start >= length or end is None or end < start:
+					server.send_response(416)
+					server.send_header("Accept-Ranges", "bytes")
+					server.send_header("Content-Range", f"bytes */{length}")
+					server.end_headers()
+					return
+				if end >= length:
+					end = length - 1
+				chunk_length = end - start + 1
+				server.send_response(206)
+				server.send_header("Accept-Ranges", "bytes")
+				server.send_header("Content-Range", f"bytes {start}-{end}/{length}")
+				server.send_header("Content-Length", str(chunk_length))
+				server.send_header("Content-Type", mime)
+				f.seek(start)
+			else:
+				server.send_response(200)
+				server.send_header("Accept-Ranges", "bytes")
+				server.send_header("Content-Type", mime)
+				server.send_header("Content-Length", str(length))
+				f.seek(0)
 
-		else:
-			server.send_response(200)
-			server.send_header("Accept-Ranges", "bytes")
-			server.send_header("Content-Type", mime)
-			server.send_header("Content-Length", str(length))
+			server.end_headers()
 
+			remaining = end - start + 1 if range_req else None
+			while True:
+				read_size = 65536 if remaining is None else min(65536, remaining)
+				if read_size <= 0:
+					break
+				data = f.read(read_size)
+				if not data:
+					break
+				server.wfile.write(data)
+				if remaining is not None:
+					remaining -= len(data)
+	except OSError:
+		server.send_response(404)
 		server.end_headers()
 
-		while True:
-			data = f.read(1000)
-			if not data:
-				break
-			server.wfile.write(data)
-
 
 def webserve(
 	pctl: PlayerCtl,
@@ -363,14 +412,20 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 			callback()
 			self.wfile.write(b"OK")
 
+		def toggle_album_shuffle(self) -> None:
+			pctl.album_shuffle_mode ^= True
+			tauon.gui.update += 1
+
 		def parse_trail(self, text: str) -> tuple[list[str], dict[str, str]]:
 			params: dict[str, str] = {}
 			both = text.split("?")
 			levels = both[0].split("/")
 			if len(both) > 1:
-				pairs = both[2].split("&")
+				pairs = both[1].split("&")
 				for p in pairs:
-					aa, bb = p.split("=")
+					if "=" not in p:
+						continue
+					aa, bb = p.split("=", 1)
 					params[aa] = bb
 
 			return levels, params
@@ -381,7 +436,7 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 			playlist_index: int | None = None,
 			track: TrackClass | None = None,
 			album_id: int = -1,
-		) -> None:
+		) -> dict[str, int | str | bool]:
 			if track is None:
 				if playlist_index is None:
 					playlist = pctl.multi_playlist[pctl.active_playlist_playing].playlist_ids
@@ -390,7 +445,7 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 				track_id = playlist[track_position]
 				track = pctl.get_track(track_id)
 
-			data = {}
+			data: dict[str, int | str | bool] = {}
 			data["title"] = track.title
 			data["artist"] = track.artist
 			data["album"] = track.album
@@ -426,7 +481,7 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 
 			if path.startswith("/api1/pic/small/"):
 				value = path[16:]
-				if value.isalnum() and int(value) in pctl.master_library:
+				if value.isdigit() and int(value) in pctl.master_library:
 					track = pctl.get_track(int(value))
 					raw = album_art_gen.save_thumb(track, (75, 75), "")
 					if raw:
@@ -444,10 +499,10 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 					self.end_headers()
 					self.wfile.write(b"Invalid parameter")
 
-			if path.startswith("/api1/pic/medium/"):
+			elif path.startswith("/api1/pic/medium/"):
 				value = path[17:]
 				logging.info(value)
-				if value.isalnum() and int(value) in pctl.master_library:
+				if value.isdigit() and int(value) in pctl.master_library:
 					track = pctl.get_track(int(value))
 					raw = album_art_gen.save_thumb(track, (1000, 1000), "")
 					if raw:
@@ -465,9 +520,9 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 					self.end_headers()
 					self.wfile.write(b"Invalid parameter")
 
-			if path.startswith("/api1/lyrics/"):
+			elif path.startswith("/api1/lyrics/"):
 				value = path[13:]
-				if value.isalnum() and int(value) in pctl.master_library:
+				if value.isdigit() and int(value) in pctl.master_library:
 					track = pctl.get_track(int(value))
 					data = {}
 					data["track_id"] = track.index
@@ -522,14 +577,23 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 
 				if param.isdigit() and int(param) in pctl.master_library:
 					track = pctl.master_library[int(param)]
-					mime = "audio/mpeg"
-					if track.file_ext == "FLAC":
-						mime = "audio/flac"
-					if track.file_ext in {"OGG", "OPUS", "OGA"}:
-						mime = "audio/ogg"
-					if track.file_ext == "M4A":
-						mime = "audio/mp4"
-					send_file(track.fullpath, mime, self)
+					if not track.fullpath or not os.path.isfile(track.fullpath):
+						self.send_response(404)
+						self.end_headers()
+						self.wfile.write(b"File unavailable")
+					else:
+						mime = "audio/mpeg"
+						if track.file_ext == "FLAC":
+							mime = "audio/flac"
+						if track.file_ext in {"OGG", "OPUS", "OGA"}:
+							mime = "audio/ogg"
+						if track.file_ext == "M4A":
+							mime = "audio/mp4"
+						send_file(track.fullpath, mime, self)
+				else:
+					self.send_response(404)
+					self.end_headers()
+					self.wfile.write(b"Invalid parameter")
 
 			elif path.startswith("/api1/start/"):
 				levels, _ = self.parse_trail(path)
@@ -555,12 +619,16 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 				self.run_command(tauon.pctl.play)
 			elif path == "/api1/pause":
 				self.run_command(tauon.pctl.pause_only)
+			elif path == "/api1/stop":
+				self.run_command(tauon.pctl.stop)
 			elif path == "/api1/next":
 				self.run_command(tauon.pctl.advance)
 			elif path == "/api1/back":
 				self.run_command(tauon.pctl.back)
 			elif path == "/api1/shuffle":
 				self.run_command(tauon.toggle_random)
+			elif path == "/api1/album-shuffle":
+				self.run_command(self.toggle_album_shuffle)
 			elif path == "/api1/repeat":
 				self.run_command(tauon.toggle_repeat)
 			elif path == "/api1/auto-stop":
@@ -753,6 +821,7 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 					"status": "stopped",
 					"inc": pctl.db_inc,
 					"shuffle": pctl.random_mode is True,
+					"album_shuffle": pctl.album_shuffle_mode is True,
 					"repeat": pctl.repeat_mode is True,
 					"progress": 0,
 					"auto_stop": tauon.pctl.stop_mode != StopMode.OFF,
@@ -778,7 +847,7 @@ def webserve2(pctl: PlayerCtl, album_art_gen: AlbumArt, tauon: Tauon) -> None:
 				data["album_id"] = 0
 				playlist = pctl.playing_playlist()
 
-				if p < len(playlist):
+				if track is not None and p < len(playlist):
 					while True:
 						if p < 0 or pctl.get_track(playlist[p]).parent_folder_path != track.parent_folder_path:
 							p += 1
@@ -846,7 +915,13 @@ def controller(tauon: Tauon) -> None:
 				tauon.toggle_repeat()
 			if path.startswith("/open/"):
 				rest = path[6:]
-				path = base64.urlsafe_b64decode(rest.encode()).decode()
+				try:
+					path = base64.urlsafe_b64decode(rest.encode()).decode()
+				except Exception:
+					logging.exception("Invalid /open/ payload")
+					self.send_response(400)
+					self.end_headers()
+					return
 				tauon.open_uri(path)
 
 			self.send_response(200)
@@ -929,7 +1004,7 @@ def stream_proxy(tauon: Tauon) -> None:
 
 					self.wfile.write(tauon.stream_proxy.chunks[position])
 
-					if tauon.prefs.backend == 4:
+					if tauon.prefs.backend == Backend.PHAZOR:
 						tauon.vb.input(tauon.stream_proxy.chunks[position])
 
 					position += 1
